@@ -64,7 +64,7 @@ label (const uint8_t * src, size_t srclen, uint8_t * dst, size_t * dstlen,
 	   entirely in lowercase (converting it to lowercase if
 	   necessary), and apply the tests of Section 5.4 and the
 	   conversion of Section 5.5 to that form. */
-	return -1;
+	return IDN2_INVALID_FLAGS;
 
       if (srclen > IDN2_LABEL_MAX_LENGTH)
 	return IDN2_TOO_BIG_LABEL;
@@ -120,15 +120,19 @@ label (const uint8_t * src, size_t srclen, uint8_t * dst, size_t * dstlen,
   (TEST_NFC | TEST_2HYPHEN | TEST_HYPHEN_STARTEND | TEST_LEADING_COMBINING | TEST_NONTRANSITIONAL)
 
 static int
-_tr46 (const uint8_t * domain_u8, uint8_t ** out, int transitional)
+_tr46 (const uint8_t * domain_u8, uint8_t ** out, int flags)
 {
   size_t len, it;
   uint32_t *domain_u32;
   int err = IDN2_OK, rc;
+  int transitional = 0;
+  int test_flags;
+
+  if (flags & IDN2_TRANSITIONAL)
+    transitional = 1;
 
   /* convert UTF-8 to UTF-32 */
-  if (!
-      (domain_u32 =
+  if (!(domain_u32 =
        u8_to_u32 (domain_u8, u8_strlen (domain_u8) + 1, NULL, &len)))
     {
       if (errno == ENOMEM)
@@ -173,9 +177,22 @@ _tr46 (const uint8_t * domain_u8, uint8_t ** out, int transitional)
 	  else
 	    len2++;
 	}
+      else if (!(flags & IDN2_USE_STD3_ASCII_RULES))
+        {
+	  if (map_is (&map, TR46_FLG_DISALLOWED_STD3_VALID))
+	    {
+	      /* valid because UseSTD3ASCIIRules=false, see #TR46 5 */
+	      len2++;
+	    }
+	  else if (map_is (&map, TR46_FLG_DISALLOWED_STD3_MAPPED))
+	    {
+	      /* mapped because UseSTD3ASCIIRules=false, see #TR46 5 */
+	      len2 += map.nmappings;
+	    }
+        }
     }
 
-  uint32_t *tmp = malloc ((len2 + 1) * sizeof (uint32_t));
+  uint32_t *tmp = (uint32_t *) malloc ((len2 + 1) * sizeof (uint32_t));
   if (!tmp)
     {
       free (domain_u32);
@@ -215,6 +232,17 @@ _tr46 (const uint8_t * domain_u8, uint8_t ** out, int transitional)
 	  else
 	    tmp[len2++] = c;
 	}
+      else if (!(flags & IDN2_USE_STD3_ASCII_RULES))
+        {
+	  if (map_is (&map, TR46_FLG_DISALLOWED_STD3_VALID))
+	    {
+	      tmp[len2++] = c;
+	    }
+	  else if (map_is (&map, TR46_FLG_DISALLOWED_STD3_MAPPED))
+	    {
+	      len2 += get_map_data (tmp + len2, &map);
+	    }
+        }
     }
   free (domain_u32);
 
@@ -267,16 +295,25 @@ _tr46 (const uint8_t * domain_u8, uint8_t ** out, int transitional)
 	      return rc;
 	    }
 
+	  test_flags = TR46_NONTRANSITIONAL_CHECK;
+
+	  if (!(flags & IDN2_USE_STD3_ASCII_RULES))
+	    test_flags |= TEST_ALLOW_STD3_DISALLOWED;
+
 	  if ((rc =
-	       _idn2_label_test (TR46_NONTRANSITIONAL_CHECK, name_u32,
+	       _idn2_label_test (test_flags, name_u32,
 				 name_len)))
 	    err = rc;
 	}
       else
 	{
+	  test_flags = transitional ? TR46_TRANSITIONAL_CHECK : TR46_NONTRANSITIONAL_CHECK;
+
+	  if (!(flags & IDN2_USE_STD3_ASCII_RULES))
+	    test_flags |= TEST_ALLOW_STD3_DISALLOWED;
+
 	  if ((rc =
-	       _idn2_label_test (transitional ? TR46_TRANSITIONAL_CHECK :
-				 TR46_NONTRANSITIONAL_CHECK, s, e - s)))
+	       _idn2_label_test (test_flags, s, e - s)))
 	    err = rc;
 	}
 
@@ -315,13 +352,22 @@ _tr46 (const uint8_t * domain_u8, uint8_t ** out, int transitional)
  * must be encoded in UTF-8 and be in Unicode NFC form.
  *
  * Pass %IDN2_NFC_INPUT in @flags to convert input to NFC form before
- * further processing.  Pass %IDN2_ALABEL_ROUNDTRIP in @flags to
+ * further processing.  %IDN2_TRANSITIONAL and %IDN2_NONTRANSITIONAL
+ * do already imply %IDN2_NFC_INPUT.
+ * Pass %IDN2_ALABEL_ROUNDTRIP in @flags to
  * convert any input A-labels to U-labels and perform additional
- * testing.  Pass %IDN2_TRANSITIONAL to enable Unicode TR46
+ * testing (not implemented yet).
+ * Pass %IDN2_TRANSITIONAL to enable Unicode TR46
  * transitional processing, and %IDN2_NONTRANSITIONAL to enable
  * Unicode TR46 non-transitional processing.  Multiple flags may be
- * specified by binary or:ing them together, for example
- * %IDN2_NFC_INPUT | %IDN2_NONTRANSITIONAL.
+ * specified by binary or:ing them together.
+ *
+ * After version 2.0.3: %IDN2_USE_STD3_ASCII_RULES disabled by default.
+ * Previously we were eliminating non-STD3 characters from domain strings
+ * such as _443._tcp.example.com, or IPs 1.2.3.4/24 provided to libidn2
+ * functions. That was an unexpected regression for applications switching
+ * from libidn and thus it is no longer applied by default.
+ * Use %IDN2_USE_STD3_ASCII_RULES to enable that behavior again.
  *
  * After version 0.11: @lookupname may be NULL to test lookup of @src
  * without allocating memory.
@@ -362,7 +408,7 @@ idn2_lookup_u8 (const uint8_t * src, uint8_t ** lookupname, int flags)
       uint8_t *out;
       size_t outlen;
 
-      rc = _tr46 (src, &out, tr46_mode == IDN2_TRANSITIONAL);
+      rc = _tr46 (src, &out, flags);
       if (rc != IDN2_OK)
 	return rc;
 
@@ -414,7 +460,7 @@ idn2_lookup_u8 (const uint8_t * src, uint8_t ** lookupname, int flags)
 
   if (lookupname)
     {
-      uint8_t *tmp = malloc (lookupnamelen + 1);
+      uint8_t *tmp = (uint8_t *) malloc (lookupnamelen + 1);
 
       if (tmp == NULL)
 	return IDN2_MALLOC;
@@ -528,7 +574,7 @@ idn2_to_ascii_4i (const uint32_t * input, size_t inlen, char * output, int flags
       return IDN2_OK;
     }
 
-  input_u32 = malloc ((inlen + 1) * sizeof(uint32_t));
+  input_u32 = (uint32_t *) malloc ((inlen + 1) * sizeof(uint32_t));
   if (!input_u32)
     return IDN2_MALLOC;
 
