@@ -812,6 +812,15 @@ static const struct idna idna[] = {
     "\x7e\x64\x61\x72\x10\x2f\x2f\xf9\x2b\x71\x60\x79\x7b\x2e\x63\x75\x2b\x61\x65\x72\x75\x65\x56\x66\x7f\x62\xc5\x76\xe5\x00",
     "", IDN2_ENCODING_ERROR, IDN2_NONTRANSITIONAL,
   },
+  /* √.com */
+  {"\xe2\x88\x9a.com", "xn--19g.com", IDN2_OK, IDN2_TRANSITIONAL},
+  /* domains with non-STD3 characters (removed by default when using TR46 transitional/non-trnasitional */
+  {"_443._tcp.example.com", "_443._tcp.example.com", IDN2_OK, 0},
+  {"_443._tcp.example.com", "_443._tcp.example.com", IDN2_OK, IDN2_TRANSITIONAL},
+  {"_443._tcp.example.com", "_443._tcp.example.com", IDN2_OK, IDN2_NONTRANSITIONAL},
+  {"_443._tcp.example.com", "443.tcp.example.com", IDN2_OK, IDN2_USE_STD3_ASCII_RULES|IDN2_NONTRANSITIONAL},
+  {"_443._tcp.example.com", "443.tcp.example.com", IDN2_OK, IDN2_USE_STD3_ASCII_RULES|IDN2_TRANSITIONAL},
+  {"_443._tcp.example.com", "_443._tcp.example.com", IDN2_OK, IDN2_USE_STD3_ASCII_RULES}, /* flag is ignored when not using TR46 */
 };
 
 static int ok = 0, failed = 0;
@@ -912,7 +921,7 @@ test_homebrewed(void)
   } else
     ok++;
 
-  out = (void *) 0x123;
+  out = (uint8_t *) 0x123;
   if ((rc = idn2_lookup_u8 (NULL, &out, 0)) != IDN2_OK) {
     failed++;
     printf("special #2 failed with %d\n", rc);
@@ -928,7 +937,7 @@ test_homebrewed(void)
   } else
     ok++;
 
-  out = (void *) 0x123;
+  out = (uint8_t *) 0x123;
   if ((rc = idn2_lookup_ul (NULL, (char **) &out, 0)) != IDN2_OK) {
     failed++;
     printf("special #4 failed with %d\n", rc);
